from focs._tech import *

Tech(
    name="SHP_ROOT_ARMOR",
    description="SHP_ROOT_ARMOR_DESC",
    short_description="SHIP_PART_UNLOCK_SHORT_DESC",
    category="SHIP_PARTS_CATEGORY",
    researchcost=1,
    researchturns=1,
    tags=["PEDIA_ARMOR_PART_TECHS"],
    unlock=Item(type=UnlockShipPart, name="AR_STD_PLATE"),
    graphic="icons/tech/armor_plating.png",
)
