VERY_BAD_MULTIPLIER = 0.5

BAD_MULTIPLIER = 0.75

AVERAGE_MULTIPLIER = 1.0

GOOD_MULTIPLIER = 1.5

GREAT_MULTIPLIER = 2.0

ULTIMATE_MULTIPLIER = 3.0

BAD_MULTIPLIER_WHOLE = 0

AVERAGE_MULTIPLIER_WHOLE = 1

GOOD_MULTIPLIER_WHOLE = 2

GREAT_MULTIPLIER_WHOLE = 3

ULTIMATE_MULTIPLIER_WHOLE = 4

ADEQUATE_MULTIPLIER = 0.75

POOR_MULTIPLIER = 0.5

HOSTILE_MULTIPLIER = 0.0
