from expansion_plans.expansion_plans_interface import (
    colonies_targeted,
    get_colonisable_outpost_ids,
    get_colonisable_planet_ids,
    outposts_targeted,
    update_colonisable_outpost_ids,
    update_colonisable_planet_ids,
)
