# Main ordered stages:
# 1. Effects with SetTargetPopulation
# 2. Effects with SetPopulation
# 3. Effects with SetTarget<other meters>
# 4. Effects with Set<other meters>
# 5. Final stage (max priority overrides and clean ups)

# Default priority: 100 = DEFAULT_PRIORITY = TARGET_BEFORE_SCALING_PRIORITY

# First stage: Target Population

BEFORE_ANYTHING_ELSE_PRIORITY = 1

TARGET_POPULATION_BEFORE_SCALING_PRIORITY = 13

TARGET_POPULATION_SCALING_PRIORITY = 15

TARGET_POPULATION_AFTER_SCALING_PRIORITY = 17

TARGET_POPULATION_2ND_SCALING_PRIORITY = 20

TARGET_POPULATION_AFTER_2ND_SCALING_PRIORITY = 22

TARGET_POPULATION_LAST_BEFORE_OVERRIDE_PRIORITY = 29

TARGET_POPULATION_OVERRIDE_PRIORITY = 30


# Second stage: Population

POPULATION_FIRST_PRIORITY = 40

POPULATION_DEFAULT_PRIORITY = 45

POPULATION_OVERRIDE_PRIORITY = 50


# Third stage: Other target meters

TARGET_FIRST_PRIORITY = 95

TARGET_EARLY_BEFORE_SCALING_PRIORITY = 98

DEFAULT_PRIORITY = 100

TARGET_BEFORE_SCALING_PRIORITY = DEFAULT_PRIORITY

TARGET_LATE_BEFORE_SCALING_PRIORITY = 102

TARGET_SCALING_PRIORITY = 105

TARGET_EARLY_AFTER_SCALING_PRIORITY = 108

TARGET_AFTER_SCALING_PRIORITY = 110

TARGET_LATE_AFTER_SCALING_PRIORITY = 112

TARGET_2ND_SCALING_PRIORITY = 115

TARGET_EARLY_AFTER_2ND_SCALING_PRIORITY = 118

TARGET_AFTER_2ND_SCALING_PRIORITY = 120

TARGET_LATE_AFTER_2ND_SCALING_PRIORITY = 122

TARGET_LAST_BEFORE_OVERRIDE_PRIORITY = 199

TARGET_OVERRIDE_PRIORITY = 200


# Fourth stage: current meters

EARLY_AFTER_ALL_TARGET_MAX_METERS_PRIORITY = 490

AFTER_ALL_TARGET_MAX_METERS_PRIORITY = 500

LATE_AFTER_ALL_TARGET_MAX_METERS_PRIORITY = 510


# Fifth stage: max priority overrides and cleanups

FOCUS_CHANGE_PENALTY_PRIORITY = 600

METER_OVERRIDE_PRIORITY = 700

CONCENTRATION_CAMP_PRIORITY = 800

SPECIAL_REMOVAL_PRIORITY = 900

END_CLEANUP_PRIORITY = 1000
